@@ -2,25 +2,27 @@
 
 require_once('common.php');
 
-require_once('PHPUnit/Autoload.php');
-
-require_once('Zend/XmlRpc/Client.php');
-require_once('Zend/Http/Client.php');
-require_once('Zend/Registry.php');
-
 abstract class RemoteTest extends PHPUnit_Framework_TestCase {
+    protected $server_url = 'http://change.me/to/work'; // http://orange/david/postfixadmin/xmlrpc.php';
+    protected $username = 'user@example.com';
+    protected $password = 'password1';
 
-    protected $server_url = 'http://orange/david/postfixadmin/xmlrpc.php';
-    protected $username = 'roger@example.com';
-    protected $password = 'patchthedog';
-
-    /* xmlrpc objects... */    
+    /* xmlrpc objects... */
     protected $user;
     protected $vacation;
     protected $alias;
 
     public function setUp() {
         parent::setUp();
+
+        if ($this->server_url == 'http://change.me/to/work') {
+            $this->markTestSkipped("Test skipped; Configuration change to \$this->server_url required");
+        }
+
+        if (!class_exists('Zend_XmlRpc_Client', true)) {
+            $this->markTestSkipped("Test skipped; Zend_XmlRpc_Client not found");
+        }
+
         $this->xmlrpc_client = new Zend_XmlRpc_Client($this->server_url);
         $http_client = $this->xmlrpc_client->getHttpClient();
         $http_client->setCookieJar();
@@ -28,8 +30,7 @@ abstract class RemoteTest extends PHPUnit_Framework_TestCase {
         $login_object = $this->xmlrpc_client->getProxy('login');
         $success = $login_object->login($this->username, $this->password);
 
-        if(!$success) {
-            var_dump($success);
+        if (!$success) {
             die("Failed to login to xmlrpc interface");
         }
     }
